<?php

namespace Drupal\route_path_rewrite\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\RouteBuilderInterface;
use Drupal\Core\Routing\RouteProviderInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure routes to rewrite settings on site.
 */
class RoutePathRewriteSettings extends ConfigFormBase {

  /**
   * The router service.
   *
   * @var \Drupal\Core\Routing\RouteProviderInterface
   */
  protected RouteProviderInterface $route_provider;

  /**
   * The route building service.
   *
   * @var \Drupal\Core\Routing\RouteBuilderInterface
   */
  protected RouteBuilderInterface $route_builder;

  /**
   * ToolbarSettingsForm constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Routing\RouteProviderInterface $route_provider
   *   The router service.
   * @param \Drupal\Core\Routing\RouteBuilderInterface $route_builder
   *   The route building service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, RouteProviderInterface $route_provider, RouteBuilderInterface $route_builder) {
    parent::__construct($config_factory);
    $this->route_provider = $route_provider;
    $this->route_builder = $route_builder;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('router.route_provider'),
      $container->get('router.builder')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['route_path_rewrite.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'route_path_rewrite.settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['#tree'] = TRUE;

    $form['routes_fieldset'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Routes'),
      '#description' => $this->t('To remove any Route, just delete its <b>Route</b> and <b>URL</b> input text (leave them blank).'),
      '#prefix' => '<div id="routes-fieldset-wrapper">',
      '#suffix' => '</div>',
    ];

    // Provides default value for each route in configuration.
    $config = $this->config('route_path_rewrite.settings');
    $data = $config->getRawData();

    $keys = [];
    if (!empty($data['routes_to_rewrite'])) {
      $keys = array_keys($data['routes_to_rewrite']);
    }

    $route_field = $form_state->get('num_routes') ? $form_state->get('num_routes') : count($keys);
    $form_state->set('num_routes', $route_field);

    for ($i = 0; $i < $route_field; $i++) {
      $key = $keys[$i] ?? "";
      $form['routes_fieldset'][$i] = [
        '#type' => 'fieldset',
      ];

      $form['routes_fieldset'][$i]['name'] = [
        '#type' => 'textfield',
        '#default_value' => $data['routes_to_rewrite'][$key]['name'] ?? '',
        '#title' => $this->t('Route'),
      ];

      $form['routes_fieldset'][$i]['new_path'] = [
        '#type' => 'textfield',
        '#title' => $this->t('URL'),
        '#default_value' => $data['routes_to_rewrite'][$key]['new_path'] ?? '',
      ];
    }

    $form['actions'] = [
      '#type' => 'actions',
    ];

    $form['routes_fieldset']['actions']['add_route'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add one more'),
      '#submit' => ['::addOne'],
      '#ajax' => [
        'callback' => '::addMoreCallback',
        'wrapper' => 'routes-fieldset-wrapper',
      ],
    ];

    if ($route_field > 1) {
      $form['routes_fieldset']['actions']['remove_route'] = [
        '#type' => 'submit',
        '#value' => $this->t('Remove last one'),
        '#submit' => ['::removeCallback'],
        '#ajax' => [
          'callback' => '::addMoreCallback',
          'wrapper' => 'routes-fieldset-wrapper',
        ],
      ];
    }

    $form_state->setCached(FALSE);

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritDoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);

    $values = $form_state->getValues();
    if (isset($values['routes_fieldset'])) {
      foreach ($values['routes_fieldset'] as $key => $routes) {
        if (!$this->isKeyNumAndRouteNameExist($routes, $key)) {
          continue;
        }

        if (count($this->route_provider->getRoutesByNames([$routes['name']])) === 0) {
          $form_state->setErrorByName(
            'routes_fieldset][' . $key . '][name',
            $this->t("Routes @name doesn't exits.", ['@name' => $routes['name']])
          );
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $route_to_rewrite = [];

    if (isset($values['routes_fieldset'])) {
      foreach ($values['routes_fieldset'] as $key => $routes) {
        if (!$this->isKeyNumAndRouteNameExist($routes, $key)) {
          continue;
        }

        $route_to_rewrite[$key] = $routes;
      }
    }

    $this->config('route_path_rewrite.settings')
      ->set('routes_to_rewrite', $route_to_rewrite)
      ->save();

    $this->route_builder->rebuild();

    $this->messenger()->addMessage($this->t('Routes added to configurations.'));
  }

  /**
   * Provides submit for adding new fieldset.
   */
  public function addOne(array &$form, FormStateInterface $form_state) {
    $route_field = $form_state->get('num_routes');
    $add_button = $route_field + 1;

    $form_state->set('num_routes', $add_button);
    $form_state->setRebuild();
  }

  /**
   * Provides ajax callback for fieldset.
   */
  public function addMoreCallback(array &$form, FormStateInterface $form_state) {
    return $form['routes_fieldset'];
  }

  /**
   * Provides submit for removing fieldset.
   */
  public function removeCallback(array &$form, FormStateInterface $form_state) {
    $route_field = (int) $form_state->get('num_routes');
    if ($route_field > 1) {
      $remove_button = $route_field - 1;
      $form_state->set('num_routes', $remove_button);
    }
    $form_state->setRebuild();
  }

  /**
   * Does Route name, and a new_path, exist in routes array?
   *
   * @param array $routes
   * @param int|string $key
   * @return bool
   */
  private function isKeyNumAndRouteNameExist(array &$routes, int|string $key): bool {
    return is_numeric($key) && !empty($routes['name']) && !empty($routes['new_path']);
  }

}
